// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:dotted_border/dotted_border.dart';
import 'package:flutter/material.dart' hide ButtonStyle;
import 'package:flutter/services.dart';
import 'package:gap/gap.dart';
import 'package:wyatt_bloc_helper/wyatt_bloc_helper.dart';
import 'package:wyatt_ui_components/wyatt_wyatt_ui_components.dart';
import 'package:wyatt_ui_kit/src/components/buttons/cubit/button_cubit.dart';
import 'package:wyatt_ui_kit/src/components/buttons/cubit/invalid_button_cubit.dart';
import 'package:wyatt_ui_kit/src/components/gradients/gradient_text.dart';
import 'package:wyatt_ui_kit/src/core/extensions/theme_extensions.dart';
import 'package:wyatt_ui_kit/src/core/helpers/linear_gradient_helper.dart';

class FileSelectionButtonScreen
    extends CubitScreen<InvalidButtonCubit, ButtonState> {
  const FileSelectionButtonScreen({
    this.leading,
    this.title,
    this.subTitle,
    this.disabledStyle,
    this.normalStyle,
    this.hoveredStyle,
    this.focusedStyle,
    this.tappedStyle,
    this.selectedStyle,
    this.invalidStyle,
    this.onPressed,
    this.mainAxisSize,
    super.key,
  });

  final Widget? leading;
  final TextWrapper? title;
  final TextWrapper? subTitle;

  final FileSelectionButtonStyle? disabledStyle;
  final FileSelectionButtonStyle? normalStyle;
  final FileSelectionButtonStyle? hoveredStyle;
  final FileSelectionButtonStyle? focusedStyle;
  final FileSelectionButtonStyle? tappedStyle;
  final FileSelectionButtonStyle? selectedStyle;
  final FileSelectionButtonStyle? invalidStyle;

  final void Function(ControlState state)? onPressed;

  final MainAxisSize? mainAxisSize;

  @override
  InvalidButtonCubit create(BuildContext context) => InvalidButtonCubit();

  @override
  Widget onBuild(BuildContext context, ButtonState state) {
    // Set a default style
    FileSelectionButtonStyle? style =
        normalStyle ?? const FileSelectionButtonStyle();

    switch (state.state) {
      case ControlState.disabled:
        style = disabledStyle ?? style;
        break;
      case ControlState.hovered:
        style = hoveredStyle ?? style;
        break;
      case ControlState.tapped:
        style = tappedStyle ?? style;
        break;
      case ControlState.focused:
        style = focusedStyle ?? style;
        break;
      case ControlState.normal:
        // already done
        break;
    }

    if (state.isSelected) {
      style = selectedStyle ?? style;
    }

    if (state.isInvalid) {
      style = invalidStyle ?? style;
    }

    print(state);

    return Focus(
      onFocusChange: (hasFocus) =>
          hasFocus ? bloc(context).onFocus() : bloc(context).onUnfocus(),
      onKeyEvent: (focusNode, event) {
        if (event.logicalKey == LogicalKeyboardKey.enter && state.isFocused) {
          onPressed?.call(state.state);
          bloc(context).onClickUpOut();
          return KeyEventResult.handled;
        }
        return KeyEventResult.ignored;
      },
      child: MouseRegion(
        cursor: SystemMouseCursors.click,
        onEnter: (event) {
          bloc(context).onMouseEnter();
        },
        onExit: (event) {
          bloc(context).onMouseLeave();
        },
        child: GestureDetector(
          onTapDown: (details) {
            bloc(context).onClickDown();
          },
          onTapUp: (details) {
            onPressed?.call(state.state);
            bloc(context).onClickUpIn();
          },
          onTapCancel: () {
            onPressed?.call(state.state);
            bloc(context).onClickUpOut();
          },
          child: DottedBorder(
            padding: EdgeInsets.zero,
            dashPattern: const [5, 5],
            strokeWidth: style.stroke ?? 3,
            color: style.borderColors?.color ?? context.colorScheme.primary,
            borderType: BorderType.RRect,
            radius: Radius.circular(style.radius ?? 0),
            strokeCap: StrokeCap.square,
            child: DecoratedBox(
              decoration: BoxDecoration(
                color: style.backgroundColors?.color ??
                    context.colorScheme.primary,
                // if no gradient colors => no default value
                gradient: (style.backgroundColors?.isGradient ?? false)
                    ? LinearGradient(
                        colors: style.backgroundColors!.colors,
                      )
                    : null,
                boxShadow: [
                  if (style.shadow != null) ...[style.shadow!]
                ],
                borderRadius: BorderRadius.all(
                  Radius.circular(style.radius ?? 0),
                ),
              ),
              child: ConstrainedBox(
                constraints: const BoxConstraints(
                  minWidth: 200,
                  minHeight: 50,
                ), // min sizes for Material buttons
                child: Padding(
                  padding: EdgeInsets.all(style.padding ?? 0),
                  child: Row(
                    mainAxisSize: mainAxisSize ?? MainAxisSize.min,
                    mainAxisAlignment: MainAxisAlignment.spaceBetween,
                    children: [
                      if (leading != null) ...[
                        leading ?? const SizedBox.shrink(),
                        Gap(style.padding ?? 10),
                      ],

                      // Choose color
                      // label.style.color ??
                      //  buttonStyle.foregroundColor.color ??
                      //    context.textTheme.titleLarge.color
                      //
                      // Choose gradient
                      // label.gradient ??
                      //  buttonStyle.foregroundColor.colors ??
                      //    null
                      Column(
                        mainAxisAlignment: MainAxisAlignment.spaceBetween,
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: [
                          if (title != null) ...[
                            Builder(
                              builder: (context) {
                                final color = title?.style?.color ??
                                    style?.foregroundColors?.color ??
                                    context.textTheme.titleLarge?.color;
                                final buttonStyleGradient =
                                    (style?.foregroundColors?.isGradient ??
                                            false)
                                        ? style?.foregroundColors?.colors
                                        : null;
                                final gradient =
                                    title?.gradient ?? buttonStyleGradient;

                                return Text(
                                  title!.text,
                                  style: (title!.style ??
                                          context.textTheme.titleLarge)
                                      ?.copyWith(color: color),
                                ).toGradient(
                                  LinearGradientHelper.fromNullableColors(
                                    gradient,
                                  ),
                                );
                              },
                            ),
                          ],
                          if (subTitle != null) ...[
                            Builder(
                              builder: (context) {
                                final color = subTitle?.style?.color ??
                                    style?.foregroundColors?.color ??
                                    context.textTheme.bodyMedium?.color;
                                final buttonStyleGradient =
                                    (style?.foregroundColors?.isGradient ??
                                            false)
                                        ? style?.foregroundColors?.colors
                                        : null;
                                final gradient =
                                    subTitle?.gradient ?? buttonStyleGradient;

                                return Text(
                                  subTitle!.text,
                                  style: (subTitle!.style ??
                                          context.textTheme.bodyMedium)
                                      ?.copyWith(color: color),
                                ).toGradient(
                                  LinearGradientHelper.fromNullableColors(
                                    gradient,
                                  ),
                                );
                              },
                            ),
                          ],
                        ],
                      ),
                    ],
                  ),
                ),
              ),
            ),
          ),
        ),
      ),
    );
  }
}
