// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/material.dart' hide ButtonStyle;
import 'package:flutter/services.dart';
import 'package:gap/gap.dart';
import 'package:wyatt_bloc_helper/wyatt_bloc_helper.dart';
import 'package:wyatt_ui_components/wyatt_wyatt_ui_components.dart';
import 'package:wyatt_ui_kit/src/components/buttons/cubit/button_cubit.dart';
import 'package:wyatt_ui_kit/src/components/buttons/cubit/selectable_button_cubit.dart';
import 'package:wyatt_ui_kit/src/components/gradients/gradient_box_border.dart';
import 'package:wyatt_ui_kit/src/components/gradients/gradient_text.dart';
import 'package:wyatt_ui_kit/src/core/extensions/theme_extensions.dart';
import 'package:wyatt_ui_kit/src/core/helpers/linear_gradient_helper.dart';

class SymbolButtonScreen
    extends CubitScreen<SelectableButtonCubit, ButtonState> {
  const SymbolButtonScreen({
    this.icon,
    this.label,
    this.disabledStyle,
    this.normalStyle,
    this.hoveredStyle,
    this.focusedStyle,
    this.tappedStyle,
    this.selectedStyle,
    this.onPressed,
    this.mainAxisSize,
    super.key,
  });

  final Widget? icon;
  final TextWrapper? label;

  final SymbolButtonStyle? disabledStyle;
  final SymbolButtonStyle? normalStyle;
  final SymbolButtonStyle? hoveredStyle;
  final SymbolButtonStyle? focusedStyle;
  final SymbolButtonStyle? tappedStyle;
  final SymbolButtonStyle? selectedStyle;

  final void Function(ControlState state)? onPressed;

  final MainAxisSize? mainAxisSize;

  @override
  SelectableButtonCubit create(BuildContext context) => SelectableButtonCubit();

  @override
  Widget onBuild(BuildContext context, ButtonState state) {
    // Set a default style
    SymbolButtonStyle? style = normalStyle ?? const SymbolButtonStyle();

    switch (state.state) {
      case ControlState.disabled:
        style = disabledStyle ?? style;
        break;
      case ControlState.hovered:
        style = hoveredStyle ?? style;
        break;
      case ControlState.tapped:
        style = tappedStyle ?? style;
        break;
      case ControlState.focused:
        style = focusedStyle ?? style;
        break;
      case ControlState.normal:
        break;
    }

    if (state.isSelected) {
      style = selectedStyle ?? style;
    }

    return Focus(
      onFocusChange: (hasFocus) =>
          hasFocus ? bloc(context).onFocus() : bloc(context).onUnfocus(),
      onKeyEvent: (focusNode, event) {
        if (event.logicalKey == LogicalKeyboardKey.enter && state.isFocused) {
          onPressed?.call(state.state);
          bloc(context).onClickUpOut();
          return KeyEventResult.handled;
        }
        return KeyEventResult.ignored;
      },
      child: MouseRegion(
        cursor: SystemMouseCursors.click,
        onEnter: (event) {
          bloc(context).onMouseEnter();
        },
        onExit: (event) {
          bloc(context).onMouseLeave();
        },
        child: GestureDetector(
          behavior: HitTestBehavior.opaque,
          onTapDown: (details) {
            bloc(context).onClickDown();
          },
          onTapUp: (details) {
            onPressed?.call(state.state);
            bloc(context).onClickUpIn();
          },
          onTapCancel: () {
            onPressed?.call(state.state);
            bloc(context).onClickUpOut();
          },
          child: Row(
            mainAxisSize: mainAxisSize ?? MainAxisSize.min,
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: [
              SizedBox.square(
                dimension: style.dimension ?? 60,
                child: AspectRatio(
                  aspectRatio: 1,
                  child: DecoratedBox(
                    decoration: BoxDecoration(
                      color: style.backgroundColors?.color ??
                          context.colorScheme.primary,
                      // If no border color => no default value
                      border: (style.borderColors != null)
                          ? (style.borderColors?.isGradient ?? false)
                              ? GradientBoxBorder(
                                  gradient: LinearGradient(
                                    colors: style.borderColors!.colors,
                                  ),
                                  width: style.stroke ?? 2,
                                )
                              : Border.all(
                                  color: style.borderColors!.color,
                                  width: style.stroke ?? 2,
                                )
                          : null,
                      // if no gradient colors => no default value
                      gradient: (style.backgroundColors?.isGradient ?? false)
                          ? LinearGradient(
                              colors: style.backgroundColors!.colors,
                            )
                          : null,
                      boxShadow: [
                        if (style.shadow != null) ...[style.shadow!]
                      ],
                      borderRadius: BorderRadius.all(
                        Radius.circular(style.radius ?? 0),
                      ),
                    ),
                    child: ConstrainedBox(
                      constraints: BoxConstraints(
                        minWidth: style.dimension ?? 60,
                      ), // min sizes for Material buttons
                      child: Padding(
                        padding: EdgeInsets.all(style.padding ?? 0),
                        child: Center(
                          // Choose color
                          // button.foreground.colors (gradient) ??
                          //  buttonStyle.foregroundColor.color ??
                          //    context.colorScheme.secondary
                          child: Builder(
                            builder: (context) {
                              final gradient =
                                  (style?.foregroundColors?.isGradient ?? false)
                                      ? LinearGradient(
                                          colors:
                                              style!.foregroundColors!.colors,
                                        )
                                      : null;
                              final color = style?.foregroundColors?.color ??
                                  context.colorScheme.secondary;
                              if (gradient != null) {
                                return ShaderMask(
                                  blendMode: BlendMode.srcIn,
                                  shaderCallback: (bounds) =>
                                      gradient.createShader(
                                    Rect.fromLTWH(
                                      0,
                                      0,
                                      bounds.width,
                                      bounds.height,
                                    ),
                                  ),
                                  child: icon,
                                );
                              }
                              return ColorFiltered(
                                colorFilter:
                                    ColorFilter.mode(color, BlendMode.srcIn),
                                child: icon,
                              );
                            },
                          ),
                        ),
                      ),
                    ),
                  ),
                ),
              ),
              // Choose color
              // label.style.color ??
              //  buttonStyle.foregroundColor.color ??
              //    context.textTheme.titleLarge.color
              //
              // Choose gradient
              // label.gradient ??
              //  buttonStyle.foregroundColor.colors ??
              //    null
              if (label != null) ...[
                const Gap(10),
                Builder(
                  builder: (context) {
                    final color = label?.style?.color ??
                        style?.foregroundColors?.color ??
                        context.textTheme.titleLarge?.color;
                    final buttonStyleGradient =
                        (style?.foregroundColors?.isGradient ?? false)
                            ? style?.foregroundColors?.colors
                            : null;
                    final gradient = label?.gradient ?? buttonStyleGradient;

                    return Text(
                      label!.text,
                      style: (label!.style ?? context.textTheme.titleMedium)
                          ?.copyWith(color: color),
                    ).toGradient(
                      LinearGradientHelper.fromNullableColors(
                        gradient,
                      ),
                    );
                  },
                ),
              ],
            ],
          ),
        ),
      ),
    );
  }
}
